@@ -78,16 +78,14 @@ class HgignoreDialog(QDialog):
         split = QSplitter()
         vbox.addWidget(split, 1)
 
-        ignoregb = QFrame()
-        ignoregb.setFrameStyle(QFrame.Panel|QFrame.Raised)
+        ignoregb = QGroupBox()
         ivbox = QVBoxLayout()
         ignoregb.setLayout(ivbox)
         lbl = QLabel(_('<b>Ignore Filter</b>'))
         ivbox.addWidget(lbl)
         split.addWidget(ignoregb)
 
-        unknowngb = QFrame()
-        unknowngb.setFrameStyle(QFrame.Panel|QFrame.Raised)
+        unknowngb = QGroupBox()
         uvbox = QVBoxLayout()
         unknowngb.setLayout(uvbox)
         lbl = QLabel(_('<b>Untracked Files</b>'))
@@ -103,6 +101,7 @@ class HgignoreDialog(QDialog):
         unknownlist.currentTextChanged.connect(self.setGlobFilter)
         unknownlist.setContextMenuPolicy(Qt.CustomContextMenu)
         unknownlist.customContextMenuRequested.connect(self.menuRequest)
+        unknownlist.itemDoubleClicked.connect(self.unknownDoubleClicked)
         lbl = QLabel(_('Backspace or Del to remove row(s)'))
         ivbox.addWidget(lbl)
 
@@ -161,6 +160,7 @@ class HgignoreDialog(QDialog):
             base, ext = os.path.splitext(local)
             if ext:
                 filters.append(['*'+ext])
+                filters.append(['**'+ext])
         else:
             filters.append(selected)
         for f in filters:
@@ -170,6 +170,9 @@ class HgignoreDialog(QDialog):
             a.triggered.connect(self.insertFilters)
         self.contextmenu.exec_(point)
 
+    def unknownDoubleClicked(self, item):
+        self.insertFilters([str(item.text())])
+
     def insertFilters(self, pats=False, isregexp=False):
         if pats is False:
             pats = self.sender()._patterns
